package sun.print;

import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;
import java.awt.print.PrinterException;
import sun.awt.image.ByteComponentRaster;
import java.awt.geom.Area;
import java.awt.image.BufferedImage;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import java.awt.Image;
import java.awt.font.TextLayout;
import java.awt.font.FontRenderContext;
import java.awt.Font;
import java.awt.geom.AffineTransform;
import java.awt.Color;
import java.awt.Shape;
import java.awt.Graphics;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterJob;
import java.awt.Graphics2D;

class PSPathGraphics extends PathGraphics
{
    private static final int DEFAULT_USER_RES = 72;
    
    PSPathGraphics(Graphics2D graphics2D, PrinterJob printerJob, Printable printable, PageFormat pageFormat, int n, boolean b) {
        super(graphics2D, printerJob, printable, pageFormat, n, b);
    }
    
    @Override
    public Graphics create() {
        return new PSPathGraphics((Graphics2D)this.getDelegate().create(), this.getPrinterJob(), this.getPrintable(), this.getPageFormat(), this.getPageIndex(), this.canDoRedraws());
    }
    
    @Override
    public void fill(Shape shape, Color color) {
        this.deviceFill(shape.getPathIterator(new AffineTransform()), color);
    }
    
    @Override
    public void drawString(String s, int n, int n2) {
        this.drawString(s, (float)n, (float)n2);
    }
    
    @Override
    public void drawString(String s, float n, float n2) {
        this.drawString(s, n, n2, this.getFont(), this.getFontRenderContext(), 0.0f);
    }
    
    @Override
    protected boolean canDrawStringToWidth() {
        return true;
    }
    
    @Override
    protected int platformFontCount(Font font, String s) {
        return ((PSPrinterJob)this.getPrinterJob()).platformFontCount(font, s);
    }
    
    @Override
    protected void drawString(String string, float x, float y, Font font, FontRenderContext frc, float n) {
        if (string.length() == 0) {
            return;
        }
        if (font.hasLayoutAttributes() && !this.printingGlyphVector) {
            new TextLayout(string, font, frc).draw(this, x, y);
            return;
        }
        Font font2 = this.getFont();
        if (!font2.equals(font)) {
            this.setFont(font);
        }
        else {
            font2 = null;
        }
        boolean textOut = false;
        float a = 0.0f;
        float a2 = 0.0f;
        int transformed = this.getFont().isTransformed();
        if (transformed != 0) {
            AffineTransform transform = this.getFont().getTransform();
            if (transform.getType() == 1) {
                a = (float)transform.getTranslateX();
                a2 = (float)transform.getTranslateY();
                if ((double)Math.abs(a) < 1.0E-5) {
                    a = 0.0f;
                }
                if ((double)Math.abs(a2) < 1.0E-5) {
                    a2 = 0.0f;
                }
                transformed = 0;
            }
        }
        int n2 = transformed == 0;
        if (!PSPrinterJob.shapeTextProp && n2) {
            PSPrinterJob psPrinterJob = (PSPrinterJob)this.getPrinterJob();
            if (psPrinterJob.setFont(this.getFont())) {
                try {
                    psPrinterJob.setColor((Color)this.getPaint());
                }
                catch (ClassCastException ex) {
                    if (font2 != null) {
                        this.setFont(font2);
                    }
                    throw new IllegalArgumentException("Expected a Color instance");
                }
                psPrinterJob.setTransform(this.getTransform());
                psPrinterJob.setClip(this.getClip());
                textOut = psPrinterJob.textOut(this, string, x + a, y + a2, font, frc, n);
            }
        }
        if (textOut == 0) {
            if (font2 != null) {
                this.setFont(font2);
                font2 = null;
            }
            super.drawString(string, x, y, font, frc, n);
        }
        if (font2 != null) {
            this.setFont(font2);
        }
        return;
    }
    
    @Override
    protected boolean drawImageToPlatform(Image image, AffineTransform tx, Color white, int n, int n2, int n3, int n4, boolean b) {
        BufferedImage bufferedImage = this.getBufferedImage(image);
        if (bufferedImage == null) {
            return true;
        }
        PSPrinterJob psPrinterJob = (PSPrinterJob)this.getPrinterJob();
        AffineTransform transform = this.getTransform();
        if (tx == null) {
            tx = new AffineTransform();
        }
        transform.concatenate(tx);
        double[] array = new double[6];
        transform.getMatrix(array);
        Point2D.Float o = new Point2D.Float(1.0f, 0.0f);
        Point2D.Float o2 = new Point2D.Float(0.0f, 1.0f);
        transform.deltaTransform(o, o);
        transform.deltaTransform(o2, o2);
        Point2D.Float o3 = new Point2D.Float(0.0f, 0.0f);
        double distance = o.distance(o3);
        double distance2 = o2.distance(o3);
        double xRes = psPrinterJob.getXRes();
        double yRes = psPrinterJob.getYRes();
        double n5 = xRes / 72.0;
        double n6 = yRes / 72.0;
        if ((transform.getType() & 0x30) != 0x0) {
            if (distance > n5) {
                distance = n5;
            }
            if (distance2 > n6) {
                distance2 = n6;
            }
        }
        if (distance != 0.0 && distance2 != 0.0) {
            AffineTransform o4 = new AffineTransform(array[0] / distance, array[1] / distance2, array[2] / distance, array[3] / distance2, array[4] / distance, array[5] / distance2);
            Shape transformedShape = o4.createTransformedShape(new Rectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4));
            Rectangle2D bounds2D = transformedShape.getBounds2D();
            bounds2D.setRect(bounds2D.getX(), bounds2D.getY(), bounds2D.getWidth() + 0.001, bounds2D.getHeight() + 0.001);
            int n7 = (int)bounds2D.getWidth();
            int n8 = (int)bounds2D.getHeight();
            if (n7 > 0 && n8 > 0) {
                boolean n9 = true;
                if (!b && this.hasTransparentPixels(bufferedImage)) {
                    n9 = false;
                    if (this.isBitmaskTransparency(bufferedImage)) {
                        if (white == null) {
                            if (this.drawBitmaskImage(bufferedImage, tx, white, n, n2, n3, n4)) {
                                return true;
                            }
                        }
                        else if (white.getTransparency() == 1) {
                            n9 = true;
                        }
                    }
                    if (!this.canDoRedraws()) {
                        n9 = true;
                    }
                }
                else {
                    white = null;
                }
                if ((n + n3 > bufferedImage.getWidth(null) || n2 + n4 > bufferedImage.getHeight(null)) && this.canDoRedraws()) {
                    n9 = false;
                }
                if (n9 == 0) {
                    transform.getMatrix(array);
                    AffineTransform o7 = new AffineTransform(array[0] / n5, array[1] / n6, array[2] / n5, array[3] / n6, array[4] / n5, array[5] / n6);
                    Rectangle2D bounds2D2 = transform.createTransformedShape(new Rectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4)).getBounds2D();
                    bounds2D2.setRect(bounds2D2.getX(), bounds2D2.getY(), bounds2D2.getWidth() + 0.001, bounds2D2.getHeight() + 0.001);
                    int n10 = (int)bounds2D2.getWidth();
                    int n11 = (int)bounds2D2.getHeight();
                    int n12 = n10 * n11 * 3;
                    int n13 = 8388608;
                    yRes = ((xRes < yRes) ? xRes : yRes);
                    int n14 = (int)yRes;
                    double n15 = 1.0;
                    double n16 = (double)n10 / (double)n7;
                    double n17 = (double)n11 / (double)n8;
                    n16 = ((n16 > n17) ? n17 : n16);
                    int n18 = (int)((double)n14 / n16);
                    if (n18 < 72) {
                        n18 = 72;
                    }
                    while (n12 > n13 && n14 > n18) {
                        n15 *= 2.0;
                        n14 /= 2;
                        n12 /= 4;
                    }
                    if (n14 < n18) {
                        n15 = yRes / (double)n18;
                    }
                    bounds2D2.setRect(bounds2D2.getX() / n15, bounds2D2.getY() / n15, bounds2D2.getWidth() / n15, bounds2D2.getHeight() / n15);
                    psPrinterJob.saveState(this.getTransform(), this.getClip(), bounds2D2, n15, n15);
                    return true;
                }
                else {
                    BufferedImage o5 = new BufferedImage((int)bounds2D.getWidth(), (int)bounds2D.getHeight(), 5);
                    Graphics2D graphics = o5.createGraphics();
                    graphics.clipRect(0, 0, o5.getWidth(), o5.getHeight());
                    graphics.translate(-bounds2D.getX(), -bounds2D.getY());
                    graphics.transform(o4);
                    if (white == null) {
                        white = Color.white;
                    }
                    graphics.drawImage(bufferedImage, n, n2, n + n3, n2 + n4, n, n2, n + n3, n2 + n4, white, null);
                    Shape clip = this.getClip();
                    Shape transformedShape2 = this.getTransform().createTransformedShape(clip);
                    Area clip2 = new Area(AffineTransform.getScaleInstance(distance, distance2).createTransformedShape(transformedShape));
                    clip2.intersect(new Area(transformedShape2));
                    psPrinterJob.setClip(clip2);
                    Rectangle2D.Float o6 = new Rectangle2D.Float((float)(bounds2D.getX() * distance), (float)(bounds2D.getY() * distance2), (float)(bounds2D.getWidth() * distance), (float)(bounds2D.getHeight() * distance2));
                    psPrinterJob.drawImageBGR(((ByteComponentRaster)o5.getRaster()).getDataStorage(), o6.x, o6.y, (float)Math.rint((double)o6.width + 0.5), (float)Math.rint((double)o6.height + 0.5), 0.0f, 0.0f, (float)o5.getWidth(), (float)o5.getHeight(), o5.getWidth(), o5.getHeight());
                    psPrinterJob.setClip(this.getTransform().createTransformedShape(clip));
                    graphics.dispose();
                }
            }
        }
        return true;
    }
    
    @Override
    public void redrawRegion(Rectangle2D rectangle2D, double n, double n2, Shape pSrc, AffineTransform affineTransform) throws PrinterException {
        PSPrinterJob psPrinterJob = (PSPrinterJob)this.getPrinterJob();
        Printable printable = this.getPrintable();
        PageFormat pageFormat = this.getPageFormat();
        int pageIndex = this.getPageIndex();
        BufferedImage bufferedImage = new BufferedImage((int)rectangle2D.getWidth(), (int)rectangle2D.getHeight(), 5);
        Graphics2D graphics = bufferedImage.createGraphics();
        ProxyGraphics2D proxyGraphics2D = new ProxyGraphics2D(graphics, psPrinterJob);
        proxyGraphics2D.setColor(Color.white);
        proxyGraphics2D.fillRect(0, 0, bufferedImage.getWidth(), bufferedImage.getHeight());
        proxyGraphics2D.clipRect(0, 0, bufferedImage.getWidth(), bufferedImage.getHeight());
        proxyGraphics2D.translate(-rectangle2D.getX(), -rectangle2D.getY());
        proxyGraphics2D.scale((double)((float)(psPrinterJob.getXRes() / n) / 72.0f), (double)((float)(psPrinterJob.getYRes() / n2) / 72.0f));
        proxyGraphics2D.translate(-psPrinterJob.getPhysicalPrintableX(pageFormat.getPaper()) / psPrinterJob.getXRes() * 72.0, -psPrinterJob.getPhysicalPrintableY(pageFormat.getPaper()) / psPrinterJob.getYRes() * 72.0);
        proxyGraphics2D.transform(new AffineTransform(this.getPageFormat().getMatrix()));
        proxyGraphics2D.setPaint(Color.black);
        printable.print(proxyGraphics2D, pageFormat, pageIndex);
        graphics.dispose();
        psPrinterJob.setClip(affineTransform.createTransformedShape(pSrc));
        Rectangle2D.Float float1 = new Rectangle2D.Float((float)(rectangle2D.getX() * n), (float)(rectangle2D.getY() * n2), (float)(rectangle2D.getWidth() * n), (float)(rectangle2D.getHeight() * n2));
        psPrinterJob.drawImageBGR(((ByteComponentRaster)bufferedImage.getRaster()).getDataStorage(), float1.x, float1.y, float1.width, float1.height, 0.0f, 0.0f, (float)bufferedImage.getWidth(), (float)bufferedImage.getHeight(), bufferedImage.getWidth(), bufferedImage.getHeight());
    }
    
    @Override
    protected void deviceFill(PathIterator pathIterator, Color color) {
        ((PSPrinterJob)this.getPrinterJob()).deviceFill(pathIterator, color, this.getTransform(), this.getClip());
    }
    
    @Override
    protected void deviceFrameRect(int n, int n2, int n3, int n4, Color color) {
        this.draw(new Rectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4));
    }
    
    @Override
    protected void deviceDrawLine(int n, int n2, int n3, int n4, Color color) {
        this.draw(new Line2D.Float((float)n, (float)n2, (float)n3, (float)n4));
    }
    
    @Override
    protected void deviceFillRect(int n, int n2, int n3, int n4, Color color) {
        super.fill(new Rectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4));
    }
    
    @Override
    protected void deviceClip(PathIterator pathIterator) {
    }
}
